# cython: language_level=3
# cython: cdivision=True
from cpython.bytes cimport PyBytes_FromStringAndSize
from libc.stdint cimport uint8_t, uint64_t

from wyhash.backends.cython cimport wyhash


cpdef inline uint64_t hash(const uint8_t[::1] data, uint64_t seed, const uint8_t[::1] secret):
    """
    generate hash
    :param data: bytes or bytearray to hash
    :param seed: an integer
    :param secret: 32 bytes random secret
    :return: 
    """
    cdef uint64_t ret
    with nogil:
        ret = wyhash.wyhash(<void*>&data[0], <size_t>data.shape[0], seed, <uint64_t*>&secret[0])
    return ret

cpdef inline bytes make_secret(uint64_t seed):
    """
    make 32byte secret
    :param seed: 
    :return: 
    """
    cdef uint64_t _wyp[4]
    with nogil:
        wyhash.make_secret(seed, _wyp)
    return PyBytes_FromStringAndSize(<char*>_wyp, 32)



import platform
import sys

from cffi import FFI

ffibuilder = FFI()
ffibuilder.cdef(
    """
void make_secret(uint64_t seed, uint64_t *secret);
uint64_t wy2u0k(uint64_t r, uint64_t k);
double wy2gau(uint64_t r);
double wy2u01(uint64_t r);
uint64_t wyrand(uint64_t *seed);
uint64_t wyhash64(uint64_t A, uint64_t B);
uint64_t _wyp[4];
uint64_t wyhash(void *key, size_t len_, uint64_t seed, const uint64_t *secret);
    """
)

source = """
#include <stdint.h>
#include "wyhash.h"
"""

ffibuilder.set_source(
    "wyhash.backends.cffi._hash_cffi",
    source,
    sources=[],
    include_dirs=["./dep"],
)

if __name__ == "__main__":
    ffibuilder.compile()



"""
Copyright (c) 2008-2022 synodriver <synodriver@gmail.com>
"""
import os
import platform

impl = platform.python_implementation()


def _should_use_cffi() -> bool:
    ev = os.getenv("WYHASH_USE_CFFI")
    if ev is not None:
        return True
    if impl == "CPython":
        return False
    else:
        return True


if not _should_use_cffi():
    from wyhash.backends.cython import hash, make_secret
else:
    from wyhash.backends.cffi import hash, make_secret



"""
Copyright (c) 2008-2022 synodriver <synodriver@gmail.com>
"""
from wyhash.backends.cffi._hash_cffi import ffi, lib


def hash(data: bytes, seed: int, secret: bytes) -> int:
    """
    generate hash
    :param data: bytes or bytearray to hash
    :param seed: an integer
    :param secret: 32 bytes random secret
    :return:
    """
    return lib.wyhash(
        ffi.cast("void*", ffi.from_buffer(data)),
        len(data),
        seed,
        ffi.cast("uint64_t*", ffi.from_buffer(secret)),
    )


def make_secret(seed: int) -> bytes:
    """
    make 32byte secret
    :param seed:
    :return:
    """
    _wyp = ffi.new("uint64_t[4]")
    lib.make_secret(seed, _wyp)
    return ffi.unpack(ffi.cast("char*", _wyp), 32)



# -*- coding: utf-8 -*-
import glob
import os
import re
import sys
from collections import defaultdict

try:
    from Cython.Build import cythonize
except ImportError:
    Cython = None
from setuptools import Extension, find_packages, setup
from setuptools.command.build_ext import build_ext

BUILD_ARGS = defaultdict(lambda: ["-O3", "-g0"])

for compiler, args in [
    ("msvc", ["/EHsc", "/DHUNSPELL_STATIC", "/Oi", "/O2", "/Ot"]),
    ("gcc", ["-O3", "-g0"]),
]:
    BUILD_ARGS[compiler] = args


class build_ext_compiler_check(build_ext):
    def build_extensions(self):
        compiler = self.compiler.compiler_type
        args = BUILD_ARGS[compiler]
        for ext in self.extensions:
            ext.extra_compile_args = args
        super().build_extensions()


c_sources = ["wyhash/backends/cython/_hash_cy.pyx"]

extensions = [
    Extension(
        "wyhash.backends.cython._hash_cy",
        c_sources,
        include_dirs=["./dep"],
    ),
]
cffi_modules = ["wyhash/backends/cffi/build.py:ffibuilder"]


def get_dis():
    with open("README.markdown", "r", encoding="utf-8") as f:
        return f.read()


def get_version() -> str:
    path = os.path.join(
        os.path.abspath(os.path.dirname(__file__)), "wyhash", "__init__.py"
    )
    with open(path, "r", encoding="utf-8") as f:
        data = f.read()
    result = re.findall(r"(?<=__version__ = \")\S+(?=\")", data)
    return result[0]


packages = find_packages(exclude=("test", "tests.*", "test*"))


def has_option(name: str) -> bool:
    if name in sys.argv[1:]:
        sys.argv.remove(name)
        return True
    return False


setup_requires = []
install_requires = []
setup_kw = {}
if has_option("--use-cython"):
    print("building cython")
    setup_requires.append("cython")
    setup_kw["ext_modules"] = cythonize(
        extensions,
        compiler_directives={
            "cdivision": True,
            "embedsignature": True,
            "boundscheck": False,
            "wraparound": False,
        },
    )
if has_option("--use-cffi"):
    print("building cffi")
    setup_requires.append("cffi>=1.0.0")
    install_requires.append("cffi>=1.0.0")
    setup_kw["cffi_modules"] = cffi_modules


def main():
    version: str = get_version()

    dis = get_dis()
    setup(
        name="wyhash",
        version=version,
        url="https://github.com/synodriver/python-wyhash",
        packages=packages,
        keywords=["hash"],
        description="python binding for wyhash",
        long_description_content_type="text/markdown",
        long_description=dis,
        author="synodriver",
        author_email="diguohuangjiajinweijun@gmail.com",
        python_requires=">=3.6",
        setup_requires=setup_requires,
        install_requires=install_requires,
        license="BSD",
        classifiers=[
            "Development Status :: 3 - Alpha",
            "Operating System :: OS Independent",
            "License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)",
            "Programming Language :: C",
            "Programming Language :: Cython",
            "Programming Language :: Python",
            "Programming Language :: Python :: 3.6",
            "Programming Language :: Python :: 3.7",
            "Programming Language :: Python :: 3.8",
            "Programming Language :: Python :: 3.9",
            "Programming Language :: Python :: 3.10",
            "Programming Language :: Python :: Implementation :: CPython",
            "Programming Language :: Python :: Implementation :: PyPy",
        ],
        include_package_data=True,
        zip_safe=False,
        cmdclass={"build_ext": build_ext_compiler_check},
        **setup_kw
    )


if __name__ == "__main__":
    main()



# cython: language_level=3
# cython: cdivision=True
from libc.stdint cimport uint64_t


cdef extern from "wyhash.h" nogil:
    void make_secret(uint64_t seed, uint64_t *secret)
    uint64_t wy2u0k(uint64_t r, uint64_t k)
    double wy2gau(uint64_t r)
    double wy2u01(uint64_t r)
    uint64_t wyrand(uint64_t *seed)
    uint64_t wyhash64(uint64_t A, uint64_t B)
    uint64_t _wyp[4]
    uint64_t wyhash(void *key, size_t len_, uint64_t seed, const uint64_t *secret)